import IOption from './interfaces/IOption';
import IState from './interfaces/IState';

/** Calls the given function, and in case of an error, uses the given function to provide a custom description for the error message. */
function execWithCustomError(func: (...args: any[]) => any, showError: (error: Error) => string) {
  try {
    return func();
  } catch (error) {
    throw new Error(showError(error));
  }
}

/**
 * Verifies that the given value is a valid value for this option, and throws an error otherwise.
 * Also updates state variables if the value is correct.
 */
export default function verifyAndStoreOptionValue({
  option,
  value,
  verify = () => true,
  message = () => '',
  outputArgs,
  requiredOptions,
}: {
  option: string,
  value: string,
  verify: IOption['verify'],
  message: IOption['message'],
  outputArgs: IState['outputArgs'],
  requiredOptions: IState['requiredOptions'],
}): void {
  //Verify value for correctness
  const verifyValue = execWithCustomError(verify.bind(null, value), (error) => `Invalid value set for option "${option}", the validation of "${value}" failed: ${String(error)}`);
  if (verifyValue !== true) {
    const customMessage = execWithCustomError(message.bind(null, value), (error) => `Invalid value set for option "${option}": "${value}". Could not create custom error message: ${String(error)}`);
    if (customMessage !== undefined && customMessage !== '') {
      throw new Error(customMessage);
    }
    throw new Error(`Invalid value set for option "${option}": "${value}".`);
  }

  //Remember the value, and if it is a required argument, mark that we have encountered it
  outputArgs[option] = value;
  if (requiredOptions[option] === false) {
    requiredOptions[option] = true;
  }
}
